import 'package:diameter/components/detail.dart';
import 'package:diameter/components/dialogs.dart';
import 'package:diameter/components/forms.dart';
import 'package:diameter/config.dart';
import 'package:diameter/navigation.dart';
import 'package:flutter/material.dart';
import 'package:diameter/models/meal_portion_type.dart';

class MealPortionTypeDetailScreen extends StatefulWidget {
  static const String routeName = '/meal-portion-type';

  final MealPortionType? mealPortionType;

  const MealPortionTypeDetailScreen({Key? key, this.mealPortionType})
      : super(key: key);

  @override
  _MealPortionTypeDetailScreenState createState() =>
      _MealPortionTypeDetailScreenState();
}

class _MealPortionTypeDetailScreenState
    extends State<MealPortionTypeDetailScreen> {
  final GlobalKey<FormState> _mealPortionTypeForm = GlobalKey<FormState>();
  final _valueController = TextEditingController(text: '');
  final _notesController = TextEditingController(text: '');

  @override
  void initState() {
    super.initState();
    if (widget.mealPortionType != null) {
      _valueController.text = widget.mealPortionType!.value;
      _notesController.text = widget.mealPortionType!.notes ?? '';
    }
  }

  void handleSaveAction() async {
    if (_mealPortionTypeForm.currentState!.validate()) {
      bool isNew = widget.mealPortionType == null;
      isNew
          ? MealPortionType.save(
              value: _valueController.text,
              notes: _notesController.text,
            )
          : MealPortionType.update(
              widget.mealPortionType!.objectId!,
              value: _valueController.text,
              notes: _notesController.text,
            );
      Navigator.pop(context, '${isNew ? 'New' : ''} Meal Portion Type saved');
    }
  }

  void handleCancelAction() {
    bool isNew = widget.mealPortionType == null;
    if (showConfirmationDialogOnCancel &&
        ((isNew &&
                (_valueController.text != '' || _notesController.text != '')) ||
            (!isNew &&
                (_valueController.text != widget.mealPortionType!.value ||
                    _notesController.text !=
                        (widget.mealPortionType!.notes ?? ''))))) {
      Dialogs.showCancelConfirmationDialog(
        context: context,
        isNew: isNew,
        onSave: handleSaveAction,
      );
    } else {
      Navigator.pop(context);
    }
  }

  @override
  Widget build(BuildContext context) {
    bool isNew = widget.mealPortionType == null;
    return Scaffold(
      appBar: AppBar(
        title: Text(
            isNew ? 'New Meal Portion Type' : widget.mealPortionType!.value),
      ),
      drawer: const Navigation(
          currentLocation: MealPortionTypeDetailScreen.routeName),
      body: SingleChildScrollView(
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.stretch,
          children: <Widget>[
            StyledForm(
              formState: _mealPortionTypeForm,
              fields: [
                TextFormField(
                  controller: _valueController,
                  decoration: const InputDecoration(
                    labelText: 'Name',
                  ),
                  validator: (value) {
                    if (value!.trim().isEmpty) {
                      return 'Empty name';
                    }
                    return null;
                  },
                ),
                TextFormField(
                  controller: _notesController,
                  decoration: const InputDecoration(
                    labelText: 'Notes',
                    alignLabelWithHint: true,
                  ),
                  keyboardType: TextInputType.multiline,
                )
              ],
            ),
          ],
        ),
      ),
      bottomNavigationBar: DetailBottomRow(
        onCancel: handleCancelAction,
        onSave: handleSaveAction,
      ),
    );
  }
}
